@@ -1,5 +1,5 @@
 /*  Clzip - LZMA lossless data compressor
-    Copyright (C) 2010-2014 Antonio Diaz Diaz.
+    Copyright (C) 2010-2015 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -38,11 +38,10 @@ void Pp_show_msg( struct Pretty_print * const pp, const char * const msg )
     {
     if( pp->first_post )
       {
-      int i, len;
+      int i, len = pp->longest_name - strlen( pp->name );
       pp->first_post = false;
       fprintf( stderr, "  %s: ", pp->name );
-      len = pp->longest_name - strlen( pp->name );
-      for( i = 0; i < len; ++i ) fprintf( stderr, " " );
+      for( i = 0; i < len; ++i ) fputc( ' ', stderr );
       if( !msg ) fflush( stderr );
       }
     if( msg ) fprintf( stderr, "%s\n", msg );
@@ -153,7 +152,7 @@ static bool LZd_verify_trailer( struct LZ_decoder * const d,
     if( verbosity >= 0 )
       {
       Pp_show_msg( pp, 0 );
-      fprintf( stderr, "CRC mismatch; trailer says %08X, data CRC is %08X.\n",
+      fprintf( stderr, "CRC mismatch; trailer says %08X, data CRC is %08X\n",
                trailer_crc, LZd_crc( d ) );
       }
     }
@@ -164,7 +163,7 @@ static bool LZd_verify_trailer( struct LZ_decoder * const d,
     if( verbosity >= 0 )
       {
       Pp_show_msg( pp, 0 );
-      fprintf( stderr, "Data size mismatch; trailer says %llu, data size is %llu (0x%llX).\n",
+      fprintf( stderr, "Data size mismatch; trailer says %llu, data size is %llu (0x%llX)\n",
                trailer_data_size, LZd_data_position( d ), LZd_data_position( d ) );
       }
     }
@@ -175,7 +174,7 @@ static bool LZd_verify_trailer( struct LZ_decoder * const d,
     if( verbosity >= 0 )
       {
       Pp_show_msg( pp, 0 );
-      fprintf( stderr, "Member size mismatch; trailer says %llu, member size is %llu (0x%llX).\n",
+      fprintf( stderr, "Member size mismatch; trailer says %llu, member size is %llu (0x%llX)\n",
                trailer_member_size, member_size, member_size );
       }
     }
@@ -209,7 +208,7 @@ int LZd_decode_member( struct LZ_decoder * const d,
     const int pos_state = LZd_data_position( d ) & pos_state_mask;
     if( Rd_decode_bit( rdec, &d->bm_match[state][pos_state] ) == 0 )	/* 1st bit */
       {
-      const uint8_t prev_byte = LZd_get_prev_byte( d );
+      const uint8_t prev_byte = LZd_peek_prev( d );
       if( St_is_char( state ) )
         {
         state -= ( state < 4 ) ? state : 3;
@@ -221,10 +220,10 @@ int LZd_decode_member( struct LZ_decoder * const d,
         state -= ( state < 10 ) ? 3 : 6;
         LZd_put_byte( d, Rd_decode_matched( rdec,
                          d->bm_literal[get_lit_state(prev_byte)],
-                         LZd_get_byte( d, rep0 ) ) );
+                         LZd_peek( d, rep0 ) ) );
         }
       }
-    else
+    else					/* match or repeated match */
       {
       int len;
       if( Rd_decode_bit( rdec, &d->bm_rep[state] ) != 0 )	/* 2nd bit */
@@ -249,12 +248,12 @@ int LZd_decode_member( struct LZ_decoder * const d,
           {
           if( Rd_decode_bit( rdec, &d->bm_len[state][pos_state] ) == 0 )	/* 4th bit */
             { state = St_set_short_rep( state );
-              LZd_put_byte( d, LZd_get_byte( d, rep0 ) ); continue; }
+              LZd_put_byte( d, LZd_peek( d, rep0 ) ); continue; }
           }
         state = St_set_rep( state );
         len = min_match_len + Rd_decode_len( rdec, &d->rep_len_model, pos_state );
         }
-      else
+      else					/* match */
         {
         int dis_slot;
         const unsigned rep0_saved = rep0;
@@ -288,7 +287,7 @@ int LZd_decode_member( struct LZ_decoder * const d,
               if( verbosity >= 0 )
                 {
                 Pp_show_msg( pp, 0 );
-                fprintf( stderr, "Unsupported marker code '%d'.\n", len );
+                fprintf( stderr, "Unsupported marker code '%d'\n", len );
                 }
               return 4;
               }
