@@ -1,5 +1,5 @@
 /*  Clzip - LZMA lossless data compressor
-    Copyright (C) 2010-2014 Antonio Diaz Diaz.
+    Copyright (C) 2010-2015 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -256,14 +256,14 @@ struct LZ_decoder
 
 void LZd_flush_data( struct LZ_decoder * const d );
 
-static inline uint8_t LZd_get_prev_byte( const struct LZ_decoder * const d )
+static inline uint8_t LZd_peek_prev( const struct LZ_decoder * const d )
   {
   const int i = ( ( d->pos > 0 ) ? d->pos : d->buffer_size ) - 1;
   return d->buffer[i];
   }
 
-static inline uint8_t LZd_get_byte( const struct LZ_decoder * const d,
-                                    const int distance )
+static inline uint8_t LZd_peek( const struct LZ_decoder * const d,
+                                const int distance )
   {
   int i = d->pos - distance - 1;
   if( i < 0 ) i += d->buffer_size;
@@ -301,7 +301,7 @@ static inline bool LZd_init( struct LZ_decoder * const d,
   d->partial_data_pos = 0;
   d->rdec = rde;
   d->dictionary_size = dict_size;
-  d->buffer_size = max( 65536, dict_size );
+  d->buffer_size = max( 65536U, d->dictionary_size );
   d->buffer = (uint8_t *)malloc( d->buffer_size );
   if( !d->buffer ) return false;
   d->pos = 0;
@@ -319,7 +319,6 @@ static inline bool LZd_init( struct LZ_decoder * const d,
   Bm_array_init( d->bm_dis_slot[0], len_states * (1 << dis_slot_bits) );
   Bm_array_init( d->bm_dis, modeled_distances - end_dis_model );
   Bm_array_init( d->bm_align, dis_align_size );
-
   Lm_init( &d->match_len_model );
   Lm_init( &d->rep_len_model );
   d->buffer[d->buffer_size-1] = 0;		/* prev_byte of first byte */
