@@ -1,5 +1,5 @@
 /*  Clzip - LZMA lossless data compressor
-    Copyright (C) 2010-2014 Antonio Diaz Diaz.
+    Copyright (C) 2010-2015 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -227,10 +227,10 @@ static inline bool Fh_set_dictionary_size( File_header data, const unsigned sz )
     if( sz > min_dictionary_size )
       {
       const unsigned base_size = 1 << data[5];
-      const unsigned wedge = base_size / 16;
+      const unsigned fraction = base_size / 16;
       int i;
       for( i = 7; i >= 1; --i )
-        if( base_size - ( i * wedge ) >= sz )
+        if( base_size - ( i * fraction ) >= sz )
           { data[5] |= ( i << 5 ); break; }
       }
     return true;
@@ -286,8 +286,8 @@ extern int verbosity;
 void cleanup_and_fail( const int retval );
 void show_error( const char * const msg, const int errcode, const bool help );
 void internal_error( const char * const msg );
-struct Matchfinder;
+struct Matchfinder_base;
 void show_progress( const unsigned long long partial_size,
-                    const struct Matchfinder * const m,
+                    const struct Matchfinder_base * const m,
                     struct Pretty_print * const p,
                     const unsigned long long cfile_size );
